-----------------------------------------------------------------------------------------------
-- Client Lua Script for HousingToolboxPro
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "HousingLib"
 
-----------------------------------------------------------------------------------------------
-- HousingToolboxPro Module Definition
-----------------------------------------------------------------------------------------------
local HousingToolboxPro = {} 

local sin = math.sin
local cos = math.cos
local rad = math.rad
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local knSaveVersion = 1
local knTimerDelay = 1.0
local knHousingExteriorPlacementMinHeight = -60 -- Copied from the Housing addon

-----------------------------------------------------------------------------------------------
-- Helper Functions
-----------------------------------------------------------------------------------------------
function DecorToPlayerPos(x, y, z) -- Borrowed verbatum from Katia Managan's fantastic KatiaBuilderToolkit addon
	local result = {}
	result.x = 1472 + z
	result.y = -715 + y
	result.z = 1440 - x
	return result
end
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tOptions = {}
	o.aHousingDecorate = nil
	o.fHousingCancelFreePlace = nil
	o.fHousingFreePlaceConfirm = nil
	o.decorSelect = nil
	o.nCurrentItemHandle = nil
	o.bFromCrate = false
	o.bPlacing = false
	o.bLinkLikeDecor = false
	o.Resolution = 12
	o.tCopiedDecorInfo = {}
	
	o.tOptions = {
		nSaveVersion = knSaveVersion,
		bHideDecorIcon = false,
		bAdvModeDefault = false,
	}

    return o
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro Init
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:Init()
    Apollo.RegisterAddon(self)
end
 
-----------------------------------------------------------------------------------------------
-- HousingToolboxPro OnSave
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end

	return self.tOptions
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro OnRestore
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:OnRestore(eType, tSavedData)
	if tSavedData and tSavedData.nSaveVersion == knSaveVersion then
		self.tOptions = tSavedData
	end
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro OnLoad
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("HousingToolboxPro.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro OnDocLoaded
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.Overlay = Apollo.LoadForm(self.xmlDoc, "Overlay", "InWorldHudStratum", self)
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "HousingToolboxProForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
	
		self.wndDecorName = self.wndMain:FindChild("DecorNameLabel")
		self.wndDefault = self.wndMain:FindChild("DefaultWindow")
		self.wndPrompt = self.wndMain:FindChild("PromptWindow")
		self.wndPlace = self.wndMain:FindChild("PlaceWindow")
		self.wndInvalid = self.wndMain:FindChild("InvalidWindow")
		self.wndAutoLink = self.wndMain:FindChild("AutoLinkWindow")
		
		self.wndPlaceBtn = self.wndDefault:FindChild("PlaceButton")
		self.wndSourceLabel = self.wndDefault:FindChild("SourceLabel")
		self.wndCost = self.wndDefault:FindChild("CashWindow")
		
		self.wndOptions = self.wndMain:FindChild("OptionsForm")
		self.wndOptions:Show(false)
		
		self.wndActions = self.wndMain:FindChild("ActionsForm")
		self.wndActions:Show(true)
		
		self.wndAutoGroupOptions = self.wndMain:FindChild("AutoGroupForm")
		self.wndAutoGroupOptions:Show(false)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		self.xmlDoc = nil
		
		aHousingDecorate = Apollo.GetAddon("Housing")
		if aHousingDecorate == nil then
			Print("HousingToolboxPro]: Could not find the Carbine Housing Addon!")
			return
		end
		
		-- Register handlers for events, slash commands and timer, etc.
		Apollo.RegisterEventHandler("SystemKeyDown", "OnSystemKeyDown", self)
		
		Apollo.RegisterEventHandler("HousingPanelControlOpen", 			"OnPropertyEnter", self)
	    Apollo.RegisterEventHandler("HousingPanelControlClose", 		"OnPropertyExit", self)
		Apollo.RegisterEventHandler("HousingExitEditMode", 				"OnExitEditMode", self)
		Apollo.RegisterEventHandler("HousingEditModeChanged", 			"OnExitEditMode", self)
		Apollo.RegisterEventHandler("HousingFreePlaceDecorCancelled", 	"OnFreePlaceDecorCancelled", self)
		Apollo.RegisterEventHandler("HousingFreePlaceDecorQuery", 		"OnFreePlaceDecorQuery", self)
		Apollo.RegisterEventHandler("HousingFreePlaceDecorSelected", 	"OnFreePlaceDecorSelected", self)
		Apollo.RegisterEventHandler("HousingMyResidenceDecorChanged", 	"OnDecorChanged", self)
		Apollo.RegisterTimerHandler("HousingDecorIconTimer", 			"OnDecorIconUpdate", self)
		
		
		-- Hijack the default Housing addon's OnCancelFreePlace function
		fHousingCancelFreePlace = aHousingDecorate.OnCancelFreePlace
		aHousingDecorate.OnCancelFreePlace = HousingToolboxPro.OnCancelFreePlace
		
		-- Hijack the default Housing addon's OnFreePlaceConfirmBtn function
		fHousingFreePlaceConfirm = aHousingDecorate.OnFreePlaceConfirmBtn
		aHousingDecorate.OnFreePlaceConfirmBtn = HousingToolboxPro.OnFreePlaceConfirmBtn
		
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
		self:OnWindowManagementReady()
	end
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro OnWindowManagementReady
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementRegister", {strName = "HousingToolboxPro", nSaveVersion = 1})
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "HousingToolboxPro"})
	
	local hideDecorBtn = self.wndOptions:FindChild("HideIconBtn")
	hideDecorBtn:SetCheck(self.tOptions.bHideDecorIcon)
	
	local advModeBtn = self.wndOptions:FindChild("AdvModeBtn")
	advModeBtn:SetCheck(self.tOptions.bAdvModeDefault)
	
	if self.tOptions.bHideDecorIcon then
		aHousingDecorate.wndDecorIconFrame:SetScale(0.0)
	else
		aHousingDecorate.wndDecorIconFrame:SetScale(1.0)
	end
	
	if self.tOptions.bAdvModeDefault and HousingLib.GetResidence() ~= nil then
		HousingLib.GetResidence():SetCustomizationMode(HousingLib.ResidenceCustomizationMode.Advanced)
	end
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro OnSystemKeyDown
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:OnSystemKeyDown(Key)
	if self.bOnProperty == false or self.decorSelect == nil then 
		return 
	end
	
	--Print("OnSystemKeyDown: " .. Key)
	if self.bPlacing == false then
		if Apollo.IsControlKeyDown() then
			if Key == 68 then -- Ctrl+D
				self:OnDuplicatePressed()
			elseif Key == 67 then -- Ctrl+C
				aHousingDecorate:OnFreePlaceCopyBtn()
			elseif Key == 86 then -- Ctrl+V
				aHousingDecorate:OnFreePlacePasteBtn()
			elseif Key == 71 then -- Ctrl+G
				self:OnFreePlaceLinkBtn()
			end
		elseif Apollo.IsShiftKeyDown() then
			if Key == 71 then -- Shift+G
				if self.decorSelect:IsChild() then
					self:OnFreePlaceUnlinkBtn()
				else
					self:OnFreePlaceUnlinkAllBtn()
				end
			end
		else
			if Key == 8 then -- Backspace
				self:OnFreePlaceCrateBtn()
			end
		end
	end
	
	local tScreenInfo = self.decorSelect:GetDecorIconInfo()
	
	if Key == 107 or Key == 187 then -- +/Numpad+
		self.decorSelect:SetScale(tScreenInfo.fScaleCurrent + 0.1)
	elseif Key == 109 or Key == 189 then -- -/Numpad-
		self.decorSelect:SetScale(tScreenInfo.fScaleCurrent - 0.1)
	end
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro OnNextFrame
-----------------------------------------------------------------------------------------------
function HousingToolboxPro:OnNextFrame()
	self.player = GameLib.GetPlayerUnit()
	
	if self.bOnProperty == false or self.player == nil then return end
	
	self.Overlay:DestroyAllPixies()
	
	if self.decorSelect == nil or self.wndAutoLink:IsVisible() == false then return end
	
	self:DrawAutoGroupRadius()
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxPro Functions
-----------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnPropertyEnter(idPropertyInfo, idZone, bPlayerIsInside)
	if HousingLib.IsHousingWorld() then
	    self.bOnProperty = true
	end	
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnPropertyExit()
	self.bOnProperty = false
	self:OnCancel()
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnExitEditMode()
	self:OnCancel()
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDecorSelect()	
	if self.bPlacing == true then
		return
	end
	
	if self.decorSelect == aHousingDecorate.decorPreview then
		return
	end
	
	self.wndMain:Show(true)
	aHousingDecorate.wndFreePlaceFrame:Show(true)

	self.decorSelect = aHousingDecorate.decorPreview
	if self.tOptions.bSelectParentDefault == true then
		self:SelectParentDecor(self.decorSelect)
	end
	
	Event_FireGenericEvent("SendVarToRover", "SelectedDecor", self.decorSelect, 0)
	
	local strName = self.decorSelect:GetName()
	self.wndDecorName:SetText(strName)
	
	local tScreenInfo = self.decorSelect:GetDecorIconInfo()
	
	local wndSliderBar = self.wndActions:FindChild("SliderBar")
    wndSliderBar:SetMinMax(tScreenInfo.fScaleMin * 100, tScreenInfo.fScaleMax * 100)
    wndSliderBar:SetValue(tScreenInfo.fScaleCurrent * 100)
	
	self:UpdateDuplicateWindow()
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:UpdateDuplicateWindow()
	if self.decorSelect == nil then return end
	
	self.wndInvalid:Show(false)
	self.wndPlace:Show(false)
	self.wndAutoLink:Show(false)
	self.wndMain:FindChild("BG_BodyFrame"):Show(true)
	
	local strName = self.decorSelect:GetName()
	if self:IsDecorAvailableInCrate(strName) then
		self.wndDefault:Show(true)
		self.wndCost:Show(false)
		self.wndPlaceBtn:SetText("Place Duplicate")
		
		local tCrateDecorData = self:GetCrateDecorData(strName)
		if tCrateDecorData ~= nil and tCrateDecorData ~= 0 then
			local nCrateCount = #tCrateDecorData.tDecorItems
			self.wndSourceLabel:SetText("In Crate: " .. nCrateCount)
		end
	elseif self:IsDecorAvailableOnVendor(strName) then
		self.wndDefault:Show(true)
		self.wndCost:Show(true)
		self.wndPlaceBtn:SetText("Purchase Duplicate")
		self.wndSourceLabel:SetText("Vendor Cost:")
		self.wndPlaceBtn:Enable(false)
		
		local tDecorVendorData = self:GetVendorDecorData(strName)
		if tDecorVendorData ~= nil and tDecorVendorData ~= 0 then
			local nCost = tDecorVendorData.nCost;
			local eCurrencyType = tDecorVendorData.eCurrencyType
			local eGroupCurrencyType = tDecorVendorData.eGroupCurrencyType
	        local playerCurrency = GameLib.GetPlayerCurrency(eCurrencyType, eGroupCurrencyType):GetAmount()
			self.wndCost:SetMoneySystem(eCurrencyType, eGroupCurrencyType)
			self.wndCost:SetAmount(nCost, true)
			if nCost <= playerCurrency then
				self.wndPlaceBtn:Enable(true)
			end
		end
	else
		self.wndDefault:Show(false)
		self.wndInvalid:Show(true)
	end
	
	self:UpdateActionButtons()
    
    if self.decorSelect:IsParent() then
        self.wndActions:FindChild("UnlinkAllBtn"):Show(true)
    end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:UpdateActionButtons()
	self.wndActions:Show(true)
    self.wndActions:FindChild("UnlinkBtn"):Show(false)
    self.wndActions:FindChild("UnlinkAllBtn"):Show(false)
    self.wndActions:FindChild("RecallBtn"):Show(not self.bPlacing)
	self.wndActions:FindChild("LinkBtn"):Show(not self.bPlacing)
	self.wndActions:FindChild("AutoGroupBtn"):Show(not self.bPlacing)
	
	if self.decorSelect ~= nil and not self.bPlacing then
		if self.decorSelect:IsChild() then
			self.wndActions:FindChild("LinkBtn"):Show(false)
			self.wndActions:FindChild("UnlinkBtn"):Show(true)
		end
		
		if self.decorSelect:IsParent() then
			self.wndActions:FindChild("UnlinkAllBtn"):Show(true)
		end
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDecorDeselect()
	if self.wndMain:IsVisible() ~= true then
		return
	end

	self.decorSelect = nil
	self.wndMain:Show(false)
	self.wndDefault:Show(false)
	self.wndPlace:Show(false)
	aHousingDecorate.wndFreePlaceFrame:Show(false)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDecorPreviewCancelled()
	self.bPlacing = false
	self:OnDecorDeselect()
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnCancelFreePlace()
	if HousingToolboxProInst.decorSelect ~= nil then
		HousingToolboxProInst.decorSelect:CancelTransform()
	end
	
	fHousingCancelFreePlace(self)
	HousingToolboxProInst:OnDecorPreviewCancelled()
end


---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceConfirmBtn()
	if HousingToolboxProInst.bPlacing then
		HousingToolboxProInst:OnPlacePressed()
	else
		fHousingFreePlaceConfirm(self)
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceDecorCancelled(decorCancelled)
	HousingToolboxProInst:OnDecorPreviewCancelled()
	aHousingDecorate:ResetPopups()
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceDecorQuery()
	if self.bPlacing and self.decorSelect ~= nil then
		self:OnCancelPressed()
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceDecorSelected(decorSelection)
	if self.bPlacing and decorSelection ~= self.decorSelect then
		self:OnPlacePressed()
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDecorChanged(decorPlaced)
	if decorPlaced == self.decorSelect then
		self:OnDecorPreviewCancelled()
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDecorIconUpdate()
	if aHousingDecorate.wndToggleFrame:IsVisible() then
		HousingToolboxProInst:OnDecorSelect(HousingToolboxProInst)
	else
		HousingToolboxProInst:OnDecorDeselect(HousingToolboxProInst)
	end
	
	if (aHousingDecorate.decorPreview == nil or aHousingDecorate.decorPreview == 0) then
		-- Preview interrupted (by opening vendor/crate etc)
		self:OnDecorPreviewCancelled(HousingToolboxProInst)
	end 
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:IsDecorAvailable(strDecorName)
	-- first check the crate
    if self:IsDecorAvailableInCrate(strDecorName) == true then
        return true
    end    
	
	-- if they dont have that item in their crate, check if it's purchaseable from the vendor
    if self:IsDecorAvailableOnVendor(strDecorName) == true then
        return true
    end   
	
	return false
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:IsDecorAvailableInCrate(strDecorName)
	local tDecorCrateList = HousingLib.GetResidence():GetDecorCrateList()
	if tDecorCrateList ~= nil then
		for idx = 1, #tDecorCrateList do
			local tCratedDecorData = tDecorCrateList[idx]
			if tCratedDecorData.strName == strDecorName then
				return true
			end
		end
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:IsDecorAvailableOnVendor(strDecorName)
	local tDecorVendorList = HousingLib.GetDecorCatalogList()
	if tDecorVendorList ~= nil then
		for idx = 1, #tDecorVendorList do
			local tVendorDecorData = tDecorVendorList[idx]
			if tVendorDecorData.strName == strDecorName then
				return true
			end
		end
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:GetCrateDecorData(strDecorName)
	local tDecorCrateList = HousingLib.GetResidence():GetDecorCrateList()
	if tDecorCrateList ~= nil then
		for idx = 1, #tDecorCrateList do
			local tCratedDecorData = tDecorCrateList[idx]
			if tCratedDecorData.strName == strDecorName then
				return tCratedDecorData
			end
		end
	end
	
	return nil
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:GetVendorDecorData(strDecorName)
	local tDecorVendorList = HousingLib.GetDecorCatalogList()
	if tDecorVendorList ~= nil then
		for idx = 1, #tDecorVendorList do
			local tVendorDecorData = tDecorVendorList[idx]
			if tVendorDecorData.strName == strDecorName then
				return tVendorDecorData
			end
		end
	end
	
	return nil
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:SelectParentDecor(decorSelected)
	if decorSelected == nil then
		return
	end
	
	if decorSelected:GetParent() ~= nil then
		local decorParent = self:GetParentDecor(decorSelected)
		decorSelected:Deselect()
		decorParent:Select()
		self.decorSelect = decorParent
		aHousingDecorate.decorPreview = decorParent
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:GetParentDecor(decor)
	if decor:GetParent() ~= nil then
		return self:GetParentDecor(decor:GetParent())
	end
	return decor
end

-----------------------------------------------------------------------------------------------
-- HousingToolboxProForm Functions
-----------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnCancel()
	self.wndMain:Show(false)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnOptions()
	if self.wndOptions:IsVisible() then
		self.wndOptions:Show(false)
	else
		self.wndOptions:Show(true)
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnRefreshPressed()
	if self.bPlacing ~= true or self.decorSelect == nil then
		return
	end
	
	self.decorSelect:SetRotation(self.tCopiedDecorInfo.pitch, self.tCopiedDecorInfo.roll, self.tCopiedDecorInfo.yaw)
    self.decorSelect:SetPosition(self.tCopiedDecorInfo.positionX, self.tCopiedDecorInfo.positionY, self.tCopiedDecorInfo.positionZ)
    self.decorSelect:SetScale(self.tCopiedDecorInfo.scale)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDuplicatePressed()
	if aHousingDecorate.decorPreview == nil or aHousingDecorate.decorPreview == 0 then
		return
	end

	local tDecorIconInfo = aHousingDecorate.decorPreview:GetDecorIconInfo()
	local strName = aHousingDecorate.decorPreview:GetName()
	
	-- Save off the selected decor info
	self.tCopiedDecorInfo = {}
    self.tCopiedDecorInfo.name = tDecorIconInfo.strName
    self.tCopiedDecorInfo.decorIdLow = tDecorIconInfo.nDecorId
    self.tCopiedDecorInfo.decorIdHi = tDecorIconInfo.nDecorIdHi
    self.tCopiedDecorInfo.positionX = tDecorIconInfo.fWorldPosX
    self.tCopiedDecorInfo.positionY = tDecorIconInfo.fWorldPosY
    self.tCopiedDecorInfo.positionZ = tDecorIconInfo.fWorldPosZ
    self.tCopiedDecorInfo.pitch = tDecorIconInfo.fPitch
    self.tCopiedDecorInfo.roll = tDecorIconInfo.fRoll
    self.tCopiedDecorInfo.yaw = tDecorIconInfo.fYaw
    self.tCopiedDecorInfo.scale = tDecorIconInfo.fScaleCurrent

	-- first check the crate
	local tCratedDecorData = self:GetCrateDecorData(strName)
	if tCratedDecorData ~= nil then
		self.decorSelect = HousingLib.PreviewCrateDecorAtLocation(tCratedDecorData.tDecorItems[1].nDecorId, tCratedDecorData.tDecorItems[1].nDecorIdHi, tDecorIconInfo.fWorldPosX, tDecorIconInfo.fWorldPosY, tDecorIconInfo.fWorldPosZ, tDecorIconInfo.fPitch, tDecorIconInfo.fRoll, tDecorIconInfo.fYaw, tDecorIconInfo.fScaleCurrent)
		self.decorSelect:SetPosition(tDecorIconInfo.fWorldPosX, tDecorIconInfo.fWorldPosY, tDecorIconInfo.fWorldPosZ)
		self.bPlacing = true
		self.wndDefault:Show(false)
		self.wndPlace:Show(true)
		Sound.Play(Sound.PlayUIHousingHardwareAddition)
		self:UpdateActionButtons()
		return
	end
	
	-- if they dont have that item in their crate, check if it's purchaseable from the vendor
	local tVendorDecorData = self:GetVendorDecorData(strName)
	if tVendorDecorData ~= nil then
        local eCurrencyType = tVendorDecorData.eCurrencyType
        local eGroupCurrencyType = tVendorDecorData.eGroupCurrencyType
        local monCash = GameLib.GetPlayerCurrency(eCurrencyType, eGroupCurrencyType):GetAmount()
        if tVendorDecorData.nCost <= monCash then
            self.decorSelect = HousingLib.PreviewVendorDecorAtLocation(tVendorDecorData.nId, tDecorIconInfo.fWorldPosX, tDecorIconInfo.fWorldPosY, tDecorIconInfo.fWorldPosZ, tDecorIconInfo.fPitch, tDecorIconInfo.fRoll, tDecorIconInfo.fYaw, tDecorIconInfo.fScaleCurrent)
			self.decorSelect:SetPosition(tDecorIconInfo.fWorldPosX, tDecorIconInfo.fWorldPosY, tDecorIconInfo.fWorldPosZ)
			self.bPlacing = true
			self.wndDefault:Show(false)
			self.wndPlace:Show(true)
			Sound.Play(Sound.PlayUIHousingHardwareAddition)
			self:UpdateActionButtons()
            return
        else
            Print("Error! Could not purchase decor. Insufficient funds")
        end        
    end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnPlacePressed()
	aHousingDecorate.decorPreview = self.decorSelect
	fHousingFreePlaceConfirm(aHousingDecorate)
	
	self.bPlacing = false
	self.decorSelect = nil

	self.wndMain:Show(false)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnCancelPressed()
	self.bPlacing = false
	aHousingDecorate:OnCancelFreePlace()
	self:OnDecorPreviewCancelled()
	self.wndMain:Show(false)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnAutoGroupCancel( wndHandler, wndControl, eMouseButton )
	self.wndActions:Show(true)
	self.wndMain:FindChild("BG_BodyFrame"):Show(true)
	self.wndAutoLink:Show(false)
	Apollo.RemoveEventHandler("NextFrame",self)
	self.Overlay:DestroyAllPixies()
	self:UpdateDuplicateWindow()
end

---------------------------------------------------------------------------------------------------
-- OptionsForm Functions
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnOptionsClose()
	self.wndOptions:Show(false)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnHideIconCheck()
	self.tOptions.bHideDecorIcon = true
	aHousingDecorate.wndDecorIconFrame:SetScale(0.0)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnHideIconUncheck()
	self.tOptions.bHideDecorIcon = false
	aHousingDecorate.wndDecorIconFrame:SetScale(1.0)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnAdvModeCheck()
	self.tOptions.bAdvModeDefault = true
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnAdvModeUncheck()
	self.tOptions.bAdvModeDefault = false
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnSelectParentCheck()
	self.tOptions.bSelectParentDefault = true
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnSelectParentUncheck()
	self.tOptions.bSelectParentDefault = false
end


---------------------------------------------------------------------------------------------------
-- AutoGroupForm Functions
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnAutoGroupOptionsClose()
	self.wndAutoGroupOptions:Show(false)
	Apollo.RemoveEventHandler("NextFrame",self)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnLinkLikeDecorCheck()
	self.tOptions.bLinkLikeDecor = true
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnLinkLikeDecorUncheck()
	self.tOptions.bLinkLikeDecor = false
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnAutoGroupRangeChanged( wndHandler, wndControl, fNewValue, fOldValue )
	self.wndAutoLink:FindChild("RangeEditBox"):SetText(fNewValue)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnAutoGroupPressed()
	if self.decorSelect == nil then
		return
	end
	
	local fRange = tonumber(self.wndAutoLink:FindChild("RangeEditBox"):GetText())
	
	local tInfo = self.decorSelect:GetDecorIconInfo()
	
	local bInside = (tInfo.fWorldPosY < -80)

	local arDecorList = HousingLib.GetResidence():GetPlacedDecorList()
	for idx = 1, #arDecorList do
		local decor2 = arDecorList[idx]
		if decor2:GetParent() == nil and decor2 ~= self.decorSelect then
			if not self.tOptions.bLinkLikeDecor or (self.tOptions.bLinkLikeDecor and decor2:GetName() == self.decorSelect:GetName()) then
				local tInfo2 = decor2:GetDecorIconInfo()

				local fDist = math.sqrt(math.pow(tInfo2.fWorldPosX - tInfo.fWorldPosX, 2) + math.pow(tInfo2.fWorldPosY - tInfo.fWorldPosY, 2) + math.pow(tInfo2.fWorldPosZ - tInfo.fWorldPosZ, 2))
				if fDist <= fRange and ((bInside and tInfo2.fWorldPosY < -80) or (not bInside and tInfo2.fWorldPosY >= -80)) then
					decor2:Link(self.decorSelect)
				end
			end
		end
	end
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:DrawAutoGroupRadius()
	local fRange = tonumber(self.wndAutoLink:FindChild("RangeEditBox"):GetText())
	local tInfo = self.decorSelect:GetDecorIconInfo()
	local uDecorPos = DecorToPlayerPos(tInfo.fWorldPosX, tInfo.fWorldPosY, tInfo.fWorldPosZ)
	
	local resolution = 20
	local currentPoint = uDecorPos
	for i = 1, resolution do
		local rotation1 = rad(360 / (resolution/(i)))
		local rotation2 = rad(360 / (resolution/(i+1)))
		
		local startPointWorld = Vector3.New(uDecorPos.x + fRange * cos(rotation1), uDecorPos.y, uDecorPos.z + fRange * sin(rotation1))
		local endPointWorld = Vector3.New(uDecorPos.x + fRange * cos(rotation2), uDecorPos.y, uDecorPos.z + fRange * sin(rotation2))
		
		local startPoint = GameLib.WorldLocToScreenPoint(startPointWorld)
		local endPoint = GameLib.WorldLocToScreenPoint(endPointWorld)
		
		self.Overlay:AddPixie( { bLine = true, fWidth = 1,  cr = "ff0000ff", loc = { fPoints = {0,0,0,0}, nOffsets = { startPoint.x, startPoint.y, endPoint.x, endPoint.y } }    } )
	end
end


---------------------------------------------------------------------------------------------------
-- ActionsForm Functions
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceUnlinkAllBtn( wndHandler, wndControl, eMouseButton )
	aHousingDecorate:OnFreePlaceUnlinkAllBtn(wndHandler, wndControl, eMouseButton)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceLinkBtn( wndHandler, wndControl, eMouseButton )
	aHousingDecorate:OnFreePlaceLinkBtn(wndHandler, wndControl, eMouseButton)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceUnlinkBtn( wndHandler, wndControl, eMouseButton )
	aHousingDecorate:OnFreePlaceUnlinkBtn(wndHandler, wndControl, eMouseButton)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnFreePlaceCrateBtn( wndHandler, wndControl, eMouseButton )
	aHousingDecorate:OnFreePlaceCrateBtn(wndHandler, wndControl, eMouseButton)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnDecorScaleChanged( wndHandler, wndControl, fNewValue, fOldValue )
	aHousingDecorate:OnDecorScaleChanged(wndHandler, wndControl, fNewValue, fOldValue)
end

---------------------------------------------------------------------------------------------------
function HousingToolboxPro:OnAutoGroupOptionsBtn( wndHandler, wndControl, eMouseButton )
	self.wndActions:Show(false)
	self.wndDefault:Show(false)
	self.wndPlace:Show(false)
	self.wndInvalid:Show(false)
	self.wndMain:FindChild("BG_BodyFrame"):Show(false)
	
	self.wndAutoLink:FindChild("RangeEditBox"):SetText("10")
	self.wndAutoLink:Show(true)
	Apollo.RegisterEventHandler("NextFrame", "OnNextFrame", self)
end


-----------------------------------------------------------------------------------------------
-- HousingToolboxPro Instance
-----------------------------------------------------------------------------------------------
HousingToolboxProInst = HousingToolboxPro:new()
HousingToolboxProInst:Init()